/*
 * LocoTool Dynamic Runtime Linking
 *
 * Copyright (c) 2004 Thomas Johansson <prencher@prencher.dk>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * locotooldyn.c - dynamic runtime linking for locotool
 */

#include <windows.h>
#include "locotooldyn.h" 

LocoTool_Instance *LocoTool_CreateInstance(const char *dllname)
{
    LocoTool_Instance *instance;
    
    instance = calloc(sizeof(LocoTool_Instance), 1);
    if (!instance)
        return NULL;
    
    instance->module = LoadLibrary(dllname);
    if (!instance->module) {
        free(instance);
        return NULL;
    }
    
    #define GETPROC(ptr, funcptr, funcname)                                  \
    {                                                                        \
        instance->ptr = (funcptr)GetProcAddress(instance->module, funcname); \
        if (instance->ptr == NULL) {                                         \
            LocoTool_FreeInstance(instance);                                 \
            return NULL;                                                     \
        }                                                                    \
    }

    GETPROC(encode,          LocoTool_encode,          "LocoTool_encode");
    GETPROC(decode,          LocoTool_decode,          "LocoTool_decode");    
    GETPROC(set_err_handler, LocoTool_set_err_handler, "LocoTool_set_err_handler");    
    GETPROC(get_int_option,  LocoTool_get_int_option,  "LocoTool_get_int_option");
    GETPROC(set_int_option,  LocoTool_set_int_option,  "LocoTool_set_int_option");
    GETPROC(get_str_option,  LocoTool_get_str_option,  "LocoTool_get_str_option");
    GETPROC(set_str_option,  LocoTool_set_str_option,  "LocoTool_set_str_option");    
    GETPROC(rleencode,       LocoTool_rleencode,       "LocoTool_rleencode");    
    GETPROC(rledecode,       LocoTool_rledecode,       "LocoTool_rledecode");
    GETPROC(checksum,        LocoTool_checksum,        "LocoTool_checksum");
    GETPROC(decompress,      LocoTool_decompress,      "LocoTool_decompress");
    GETPROC(scramble,        LocoTool_scramble,        "LocoTool_scramble");
    GETPROC(descramble,      LocoTool_descramble,      "LocoTool_descramble");    
    GETPROC(get_version,     LocoTool_get_version,     "LocoTool_get_version");
    GETPROC(get_vernum,      LocoTool_get_vernum,      "LocoTool_get_vernum");

    #undef GETPROC
    
    return instance;
}

void LocoTool_FreeInstance(LocoTool_Instance* instance)
{
    if (instance) {
        if (instance->module)
            FreeLibrary(instance->module);
        free(instance);
    }
}
