/*
 * LocoTool Dynamic Runtime Linking
 *
 * Copyright (c) 2004 Thomas Johansson <prencher@prencher.dk>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * locotooldyn.h - dynamic runtime linking for locotool
 */

#ifndef LOCOTOOLDYN_H
#define LOCOTOOLDYN_H

#include <stdlib.h>
#ifndef _MSC_VER
#include <stdint.h>
#else
typedef unsigned __int8  uint8_t;
typedef unsigned __int32 uint32_t;
#endif

#ifdef __cplusplus
extern "C" {
#endif

// integer and string options
typedef enum {
    opt_palettetype,    // palette type to use when decoding
    opt_demofile,       // whether to generate a demo file checksum
    opt_quiet,          // no output, no message, fail silently

    opt_INTLAST,
} en_intoption;

typedef enum {
    opt_filebase,   // file path and basename for tmp/xml/dat files
                    // will have e.g. ".xml" appended to make the xml filename
    opt_pngbase,    // file path and basename for png files
                    // will have e.g. "0043.png" appended to make the png filename

    opt_STRLAST,
} en_stroption;

typedef int         (*LocoTool_encode)(char *xmlname);
typedef int         (*LocoTool_decode)(char *datname);
typedef void        (*LocoTool_err_proc)(char *message);
typedef void        (*LocoTool_set_err_handler)(LocoTool_err_proc handler);
typedef int         (*LocoTool_get_int_option)(en_stroption opt);
typedef int         (*LocoTool_set_int_option)(en_stroption opt, int val);
typedef char       *(*LocoTool_get_str_option)(en_stroption opt);
typedef char       *(*LocoTool_set_str_option)(en_stroption opt, char *str);
typedef int         (*LocoTool_rleencode)(uint8_t *data, uint32_t len, uint8_t **rleout, uint32_t *rlelen);
typedef int         (*LocoTool_rledecode)(uint8_t *data, uint32_t len, uint8_t **outdata, uint32_t *outlen);
typedef uint32_t    (*LocoTool_checksum)(uint8_t *data, uint32_t len);
typedef int         (*LocoTool_decompress)(uint8_t *data, uint32_t len, uint8_t **outdata, uint32_t *outlen);
typedef int         (*LocoTool_scramble)(uint8_t *data, uint32_t len);
typedef int         (*LocoTool_descramble)(uint8_t *data, uint32_t len);


typedef const char *(*LocoTool_get_version)();
typedef int         (*LocoTool_get_vernum)();

typedef struct {
    void *module;

    LocoTool_encode          encode;
    LocoTool_decode          decode;    
    LocoTool_set_err_handler set_err_handler;    
    LocoTool_get_int_option  get_int_option;
    LocoTool_set_int_option  set_int_option;
    LocoTool_get_str_option  get_str_option;
    LocoTool_set_str_option  set_str_option;
    LocoTool_rleencode       rleencode;    
    LocoTool_rledecode       rledecode;
    LocoTool_checksum        checksum;
    LocoTool_decompress      decompress;
    LocoTool_scramble        scramble;
    LocoTool_descramble      descramble;    
    LocoTool_get_version     get_version;
    LocoTool_get_vernum      get_vernum;
} LocoTool_Instance;

// Creates a LocoTool_Instance from the dll specified, returns NULL on failure
LocoTool_Instance *LocoTool_CreateInstance(const char *dllname);

// Unloads the module and frees the LocoTool_Instance
void LocoTool_FreeInstance(LocoTool_Instance *instance);

#ifdef __cplusplus
}
#endif

#endif
